## aggCounter.sh script
##      bash script to aggregate the SUM of BitsIn metrics for device interfaces in a group to single counter value on the device every hour
##      script has no cmdline arguments, see INPUT PARAMETER section below for hostname and credential settings, interface group and target device name
##      for changes to metric and aggregate MF names, the PARAM variable needs to be adjusted
##      csv output = CAMM engine input directory is $HOMEDIR/output

##      script is typically scheduled by cron. e.g. every hour at 05 minutes:
##5 * * * * /home/user/aggCounter.sh >> /home/user/aggCounter.log 2>&1
#
#       v1.0    August 2017, Lutz Holzbecher
#
# DISCLAIMER
# This script is provided AS IS and no warranties are provided or made.
# For more information refer to http://www.ca.com/us/legal/terms-for-user-sites.aspx
#
## INPUT PARAMETERS: set variables ==============================================================================
HOMEDIR=/home/user				# script directory
GROUPFILE=groupPM.csv			# odata query output
AGGFILE=aggregatePM.csv			# odata query output
# setup credentials user/pw
CRED='admin:admin'
DA='DAHOST'
GROUPNAME='GROUPNAME'			# interface group name
AGGHOST='DEVICENAME'    		# aggregate device name
AGGIP='DEVICEIP'				# aggregate device IP
## INPUT PARAMETERS: set variables ==============================================================================
cd $HOMEDIR
URL='http://'
URL+=$DA
URL+=':8581/odata/api/'
AGGURL=$URL
AGGURL+='devices'
GROUPURL=$URL
GROUPURL+='groups'

PARAM='$apply=aggregate(portmfs(im_BitsIn%20with%20sum%20as%20Value))&$format=text/csv&&resolution=RATE&period=1h&$select=Name,MemberCount&$filter=((Name%20eq%20%27'
PARAM+=$GROUPNAME
PARAM+='%27))'
curl -u $CRED -s -G -X GET -d $PARAM $GROUPURL  > $GROUPFILE

##curl -u admin:Hippo123 -s -G -X GET -d '$apply=aggregate(portmfs(im_Bits%20with%20sum%20as%20Value))&$format=text/csv&&resolution=RATE&period=1h&$select=Name,MemberCount&$filter=((Name%20eq%20%27SIM-group1%27))' 'http://hollu02-imdadr:8581/odata/api/groups' > $GROUPFILE
## "Name","MemberCount","portmfs/Value"
## SIM-group1,3,79200.0
#### aggregate query --------------------------------------------------------------------------------------------
sleep 2
PARAM='&resolution=RATE&period=1h&$expand=aggregates2mfs&$select=Name,aggregates2mfs/Timestamp,aggregates2mfs/im_BitsIn&$filter=((startswith(Name,%20%27'
PARAM+=$AGGHOST
PARAM+='%27)%20eq%20true))'
curl -u $CRED -s -G -X GET -d $PARAM $AGGURL  > $AGGFILE

###curl -u admin:Hippo123 -s -G -X GET -d '&resolution=RATE&period=1h&$expand=aggregates2mfs&$select=Name,aggregates2mfs/Timestamp,aggregates2mfs/im_BitsIn&$filter=((startswith(Name,%20%27HOLLU02-SIM%27)%20eq%20true))' 'http://hollu02-imdadr:8581/odata/api/devices' > $AGGFILE
## "Name","aggregates2mfs/Timestamp","aggregates2mfs/im_BitsIn"
## HOLLU02-SIM.ca.com,,

#### process query output ---------------------------------------------------------------------------------------
sleep 60
## delay write by 1min to make sure the hourly datapoint is in the last hour window of query
## time of query: x:05, last 60min
## time of data:  x:06
NOW=$(date +"%Y%m%d%H%M")
echo 'Timestamp,Device,IPAddress,Group,BitsIn' >  $HOMEDIR/output/data$NOW.csv
AggBitsInGlobal=""
{
    read
    while IFS=, read -r Device TS AggBitsIn
    do
        Timestamp=`date +%s`
        printTS=`date -d @$Timestamp "+%Y-%m-%d-%H:%M:%S" `

        AggBitsIn=${AggBitsIn::-1}
        echo "aggregate: $printTS,$AggBitsIn"
        if [ "test$AggBitsIn" == "test" ]
        then
                AggBitsIn=0.0
        fi
        ###AggBitsIn=${AggBitsIn::-2}
        ## use awk to format number as long
        AggBitsIn=`awk '{ print +$1 }' <<< $AggBitsIn`
        if [ `date "+%d-%H"` == "01-00" ]
        then
                # first day of month, first hour --> reset aggregate counter
                AggBitsIn=0
        fi
        AggBitsInGlobal=$AggBitsIn
    done
} < $AGGFILE

{
    read
    while IFS=, read -r Group MemberCount BitsIn
    do
        Timestamp=`date +%s`
        printTS=`date -d @$Timestamp  "+%Y-%m-%d-%H:%M:%S "`

        BitsIn=${BitsIn::-1}
        echo "groupSUM: $printTS,$BitsIn"
        if [ "test$BitsIn" == "test" ]
        then
                BitsIn=0.0
        fi
        BitsIn=${BitsIn::-2}
        AggBitsIn=$((AggBitsInGlobal + BitsIn))
        echo "$Timestamp,$AGGHOST,$AGGIP,$Group,$AggBitsIn" >> $HOMEDIR/output/data$NOW.csv
    done
} < $GROUPFILE
